import { __decorate, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import BaseComponent from '../BaseComponent';
import { classNamesFunction } from '@uifabric-vue/utilities';
import { getStyles } from './CalloutContent.styles';
import { concatStyleSetsWithProps } from '@uifabric/merge-styles';
import { clickedOutside } from '../../utilities/clickedOutside';
import { positionCallout, getBoundsFromTargetWindow } from '../../utilities/positioning/';
import { DirectionalHint } from '../../common/DirectionalHint';
const getClassNames = classNamesFunction();
let CalloutContentBase = class CalloutContentBase extends BaseComponent {
    constructor() {
        super(...arguments);
        this.internalKey = new Date();
        this.positions = null;
        this.positionAttempts = 0;
    }
    created() {
        window.addEventListener('click', this.onGlobalClick, true);
        window.addEventListener('scroll', this.onGlobalScroll, true);
    }
    beforeDestroy() {
        window.removeEventListener('click', this.onGlobalClick, true);
        window.removeEventListener('scroll', this.onGlobalScroll, true);
    }
    updated() {
        this.internalKey = new Date();
    }
    mounted() {
        this.updatePosition();
    }
    get classNames() {
        const { theme, className, positions, styles, calloutWidth, beakWidth } = this;
        return getClassNames(concatStyleSetsWithProps({
            theme, className, overflowYHidden: false, calloutWidth, beakWidth, backgroundColor: '#fff', calloutMaxWidth: '100%',
        }, getStyles, styles));
    }
    get actualBeakWidth() {
        return Math.sqrt(this.beakWidth * this.beakWidth * 2);
    }
    get positionCss() {
        if (!this.positions)
            return null;
        const positionCss = this.positions;
        for (const key in positionCss.elementPosition) {
            positionCss.elementPosition[key] = `${positionCss.elementPosition[key]}px`;
        }
        for (const key in positionCss.beakPosition.elementPosition) {
            positionCss.beakPosition.elementPosition[key] = `${positionCss.beakPosition.elementPosition[key]}px`;
        }
        return positionCss;
    }
    updatePosition() {
        const positions = this.positions;
        const currentProps = { ...this.$props };
        currentProps.bounds = this._getBounds();
        currentProps.target = this.target;
        const newPositions = positionCallout(currentProps, this.$refs.hostElement, this.$refs.calloutElement, this.positions);
        if ((!positions && newPositions) ||
            (positions && newPositions && !this._arePositionsEqual(positions, newPositions) && this.positionAttempts < 5)) {
            this.positionAttempts++;
            this.positions = newPositions;
        }
        else if (this.positionAttempts > 0) {
            this.positionAttempts = 0;
            this.$emit('positioned', this.positions);
        }
    }
    _getBounds() {
        if (!this._bounds) {
            const bounds = this.bounds;
            let currentBounds = typeof bounds === 'function' ? bounds(this.target, window) : bounds;
            if (!currentBounds) {
                currentBounds = getBoundsFromTargetWindow(this.target, window);
                currentBounds = {
                    top: currentBounds.top + this.minPagePadding,
                    left: currentBounds.left + this.minPagePadding,
                    right: currentBounds.right - this.minPagePadding,
                    bottom: currentBounds.bottom - this.minPagePadding,
                    width: currentBounds.width - this.minPagePadding * 2,
                    height: currentBounds.height - this.minPagePadding * 2,
                };
            }
            this._bounds = currentBounds;
        }
        return this._bounds;
    }
    onGlobalClick(e) {
        const outside = clickedOutside(e, this.$refs.calloutElement);
        if (outside)
            this.$emit('dismiss', e, 'click');
    }
    onGlobalScroll(e) {
        const outside = clickedOutside(e, this.$refs.calloutElement);
        if (outside)
            this.$emit('dismiss', e, 'scroll');
    }
    _arePositionsEqual(positions, newPosition) {
        return (this._comparePositions(positions.elementPosition, newPosition.elementPosition) &&
            this._comparePositions(positions.beakPosition.elementPosition, newPosition.beakPosition.elementPosition));
    }
    _comparePositions(oldPositions, newPositions) {
        for (const key in newPositions) {
            // This needs to be checked here and below because there is a linting error if for in does not immediately have an if statement
            if (newPositions.hasOwnProperty(key)) {
                const oldPositionEdge = oldPositions[key];
                const newPositionEdge = newPositions[key];
                if (oldPositionEdge !== undefined && newPositionEdge !== undefined) {
                    if (oldPositionEdge.toFixed(2) !== newPositionEdge.toFixed(2)) {
                        return false;
                    }
                }
                else {
                    return false;
                }
            }
        }
        return true;
    }
    render() {
        const { classNames, positionCss, isBeakVisible } = this;
        return (<div ref="hostElement" class={classNames.container}>
        <div ref="calloutElement" class={classNames.root} style={positionCss ? positionCss.elementPosition : null}>
          {isBeakVisible && (<div class={classNames.beak} style={(positionCss && positionCss.beakPosition) ? positionCss.beakPosition.elementPosition : null}/>)}
          {isBeakVisible && (<div class={classNames.beakCurtain}/>)}
          <div class={classNames.calloutMain}>
            {this.$slots.default}
          </div>
        </div>
      </div>);
    }
};
__decorate([
    Prop({ type: HTMLElement, required: true }),
    __metadata("design:type", HTMLElement)
], CalloutContentBase.prototype, "target", void 0);
__decorate([
    Prop({ type: Number, default: null }),
    __metadata("design:type", Number)
], CalloutContentBase.prototype, "calloutWidth", void 0);
__decorate([
    Prop({ type: Number, default: 16 }),
    __metadata("design:type", Number)
], CalloutContentBase.prototype, "beakWidth", void 0);
__decorate([
    Prop({ type: Number, default: 8 }),
    __metadata("design:type", Number)
], CalloutContentBase.prototype, "minPagePadding", void 0);
__decorate([
    Prop({ type: Boolean, default: true }),
    __metadata("design:type", Boolean)
], CalloutContentBase.prototype, "isBeakVisible", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], CalloutContentBase.prototype, "coverTarget", void 0);
__decorate([
    Prop({ type: Number, default: 0 }),
    __metadata("design:type", Number)
], CalloutContentBase.prototype, "gapSpace", void 0);
__decorate([
    Prop({ type: Number, default: DirectionalHint.bottomAutoEdge }),
    __metadata("design:type", Boolean)
], CalloutContentBase.prototype, "directionalHint", void 0);
__decorate([
    Prop({ type: [Object, Function], default: null }),
    __metadata("design:type", Object)
], CalloutContentBase.prototype, "bounds", void 0);
CalloutContentBase = __decorate([
    Component
], CalloutContentBase);
export { CalloutContentBase };
//# sourceMappingURL=CalloutContent.base.jsx.map