import { __decorate, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import BaseComponent from '../../BaseComponent';
import { classNamesFunction } from '@uifabric-vue/utilities';
import { DEFAULT_CELL_STYLE_PROPS } from '../DetailsRow/DetailsRow.styles';
import { Icon, FontIcon } from '../../Icon';
import { ColumnActionsMode } from '../DetailsList.types';
const MOUSEDOWN_PRIMARY_BUTTON = 0; // for mouse down event we are using ev.button property, 0 means left button
const getClassNames = classNamesFunction();
const TRANSITION_DURATION_DRAG = 200; // ms
const TRANSITION_DURATION_DROP = 1500; // ms
const CLASSNAME_ADD_INTERVAL = 20; // ms
let DetailsColumnBase = class DetailsColumnBase extends BaseComponent {
    constructor() {
        super(...arguments);
        this.ColumnActionsMode = ColumnActionsMode;
    }
    get IconComponent() {
        return this.useFastIcons ? FontIcon : Icon;
    }
    get classNames() {
        const { column, columnIndex, styles, theme, cellStyleProps = DEFAULT_CELL_STYLE_PROPS, } = this;
        return getClassNames(styles, {
            theme: theme,
            headerClassName: column.headerClassName,
            iconClassName: column.iconClassName,
            isActionable: column.columnActionsMode !== ColumnActionsMode.disabled,
            isEmpty: !column.name,
            isIconVisible: column.isSorted || column.isGrouped || column.isFiltered,
            isPadded: column.isPadded,
            isIconOnly: column.isIconOnly,
            cellStyleProps,
            transitionDurationDrag: TRANSITION_DURATION_DRAG,
            transitionDurationDrop: TRANSITION_DURATION_DROP,
        });
    }
    get columnWidth() {
        const { column, cellStyleProps } = this;
        return column.calculatedWidth +
            cellStyleProps.cellLeftPadding +
            cellStyleProps.cellRightPadding +
            (column.isPadded ? cellStyleProps.cellExtraRightPadding : 0) +
            'px';
    }
    mounted() {
        if (this.dragDropHelper && this.isDraggable) {
            this._addDragDropHandling();
        }
        const classNames = this.classNames;
        if (this.isDropped) {
            if (this.$refs.root) {
                this.$refs.root.classList.add(classNames.borderAfterDropping);
                this._async.setTimeout(() => {
                    if (this.$refs.root) {
                        this.$refs.root.classList.add(classNames.noBorderAfterDropping);
                    }
                }, CLASSNAME_ADD_INTERVAL);
            }
            this._async.setTimeout(() => {
                if (this.$refs.root) {
                    this.$refs.root.classList.remove(classNames.borderAfterDropping);
                    this.$refs.root.classList.remove(classNames.noBorderAfterDropping);
                }
            }, TRANSITION_DURATION_DROP + CLASSNAME_ADD_INTERVAL);
        }
    }
    beforeDestroy() {
        if (this._dragDropSubscription) {
            this._dragDropSubscription.dispose();
            // delete this._dragDropSubscription
        }
        this._async.dispose();
        this.events.dispose();
    }
    updated() {
        if (!this._dragDropSubscription && this.dragDropHelper && this.isDraggable) {
            this._addDragDropHandling();
        }
        if (this._dragDropSubscription && !this.isDraggable) {
            this._dragDropSubscription.dispose();
            this.events.off(this.$refs.root, 'mousedown');
            // delete this._dragDropSubscription
        }
    }
    _onColumnClick(ev) {
        const { onColumnClick, column } = this;
        if (column.columnActionsMode === ColumnActionsMode.disabled) {
            return;
        }
        if (column.onColumnClick) {
            column.onColumnClick(ev, column);
        }
        if (onColumnClick) {
            onColumnClick(ev, column);
        }
    }
    _getColumnDragDropOptions() {
        const { columnIndex } = this;
        const options = {
            selectionIndex: columnIndex,
            context: { data: columnIndex, index: columnIndex },
            canDrag: () => this.isDraggable,
            canDrop: () => false,
            onDragStart: this._onDragStart,
            updateDropState: () => undefined,
            onDrop: () => undefined,
            onDragEnd: this._onDragEnd,
        };
        return options;
    }
    _onDragStart(item, itemIndex, selectedItems, event) {
        const classNames = this.classNames;
        if (itemIndex) {
            this._updateHeaderDragInfo(itemIndex);
            this.$refs.root.classList.add(classNames.borderWhileDragging);
            this._async.setTimeout(() => {
                if (this.$refs.root) {
                    this.$refs.root.classList.add(classNames.noBorderWhileDragging);
                }
            }, CLASSNAME_ADD_INTERVAL);
        }
    }
    ;
    _onDragEnd(item, event) {
        const classNames = this.classNames;
        if (event) {
            this._updateHeaderDragInfo(-1, event);
        }
        this.$refs.root.classList.remove(classNames.borderWhileDragging);
        this.$refs.root.classList.remove(classNames.noBorderWhileDragging);
    }
    ;
    _updateHeaderDragInfo(itemIndex, event) {
        if (this.updateDragInfo) {
            this.updateDragInfo({ itemIndex }, event);
        }
    }
    ;
    _onRootMouseDown(ev) {
        const { isDraggable } = this;
        // Ignore anything except the primary button.
        if (isDraggable && ev.button === MOUSEDOWN_PRIMARY_BUTTON) {
            ev.stopPropagation();
        }
    }
    ;
    _onColumnContextMenu(ev) {
        const { onColumnContextMenu, column } = this;
        if (column.onColumnContextMenu) {
            column.onColumnContextMenu(column, ev);
            ev.preventDefault();
        }
        if (onColumnContextMenu) {
            onColumnContextMenu(column, ev);
            ev.preventDefault();
        }
    }
    ;
    _addDragDropHandling() {
        this._dragDropSubscription = this.dragDropHelper.subscribe(this.$refs.root, this.events, this._getColumnDragDropOptions());
        // We need to use native on this to prevent MarqueeSelection from handling the event before us.
        this.events.on(this.$refs.root, 'mousedown', this._onRootMouseDown);
    }
    render() {
        const { classNames, column, columnIndex, columnWidth, IconComponent, isDraggable, parentId } = this;
        return (<div ref="root" key={column.key} class={classNames.root} aria-sort={column.isSorted ? (column.isSortedDescending ? 'descending' : 'ascending') : 'none'} aria-colindex={columnIndex} role="columnheader" style={{ width: columnWidth }}>
        {isDraggable && (<IconComponent icon-name="GripperBarVertical" class={classNames.gripperBarVerticalStyle}/>)}

        <span class={classNames.cellTooltip}>
          <span id={`${parentId}-${column.key}`} aria-label={column.isIconOnly ? column.name : undefined} aria-labelledby={column.isIconOnly ? undefined : `${parentId}-${column.key}-name`} class={classNames.cellTitle} onContextmenu={this._onColumnContextMenu} onClick={this._onColumnClick}>
            <span id={`${parentId}-${column.key}-name`} class={classNames.cellName}>
              
              {(column.iconName || column.iconClassName) && (<IconComponent class={classNames.iconClassName} icon-name={column.iconName}/>)}

              
              {column.isIconOnly ? (<span class={classNames.accessibleLabel}>{column.name}</span>) : column.name}
            </span>

            
            {column.isFiltered && (<IconComponent class={classNames.nearIcon} icon-name="Filter"/>)}

            
            {column.isSorted && (<IconComponent class={classNames.sortIcon} icon-name={column.isSortedDescending ? 'SortDown' : 'SortUp'}/>)}

            
            {column.isGrouped && (<IconComponent class={classNames.nearIcon} icon-name="GroupedDescending"/>)}

            
            {column.columnActionsMode === ColumnActionsMode.hasDropdown && !column.isIconOnly && (<IconComponent icon-name="ChevronDown"/>)}
          </span>
        </span>
      </div>);
    }
};
__decorate([
    Prop({ type: Object, required: true }),
    __metadata("design:type", Object)
], DetailsColumnBase.prototype, "column", void 0);
__decorate([
    Prop({ type: Number, required: true }),
    __metadata("design:type", Number)
], DetailsColumnBase.prototype, "columnIndex", void 0);
__decorate([
    Prop({ type: Number, default: null }),
    __metadata("design:type", Number)
], DetailsColumnBase.prototype, "parentId", void 0);
__decorate([
    Prop({ type: Function, default: null }),
    __metadata("design:type", Function)
], DetailsColumnBase.prototype, "onColumnClick", void 0);
__decorate([
    Prop({ type: Function, default: null }),
    __metadata("design:type", Function)
], DetailsColumnBase.prototype, "onColumnContextMenu", void 0);
__decorate([
    Prop({ type: Object, default: null }),
    __metadata("design:type", Object)
], DetailsColumnBase.prototype, "dragDropHelper", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], DetailsColumnBase.prototype, "isDraggable", void 0);
__decorate([
    Prop({ type: Function, default: null }),
    __metadata("design:type", Function)
], DetailsColumnBase.prototype, "updateDragInfo", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], DetailsColumnBase.prototype, "isDropped", void 0);
__decorate([
    Prop({ type: Object, default: () => DEFAULT_CELL_STYLE_PROPS }),
    __metadata("design:type", Object)
], DetailsColumnBase.prototype, "cellStyleProps", void 0);
__decorate([
    Prop({ type: Boolean, default: true }),
    __metadata("design:type", Boolean)
], DetailsColumnBase.prototype, "useFastIcons", void 0);
DetailsColumnBase = __decorate([
    Component({
        components: {},
    })
], DetailsColumnBase);
export { DetailsColumnBase };
//# sourceMappingURL=DetailsColumn.base.jsx.map