import { __decorate, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import { ActionButton } from '../Button/';
import BaseComponent from '../BaseComponent';
import { Icon } from '../Icon/';
import { classNamesFunction } from '@uifabric-vue/utilities';
const getClassNames = classNamesFunction();
// The number pixels per indentation level for Nav links.
const INDENTATION_SIZE = 14;
// The number of pixels of left margin
const BASE_INDENT = 3;
export function isRelativeUrl(url) {
    // A URL is relative if it has no protocol.
    return !!url && !/^[a-z0-9+-.]:\/\//i.test(url);
}
let Nav = class Nav extends BaseComponent {
    constructor() {
        super(...arguments);
        this.isGroupCollapsed = {};
        this.internalSelectedKey = '';
    }
    render(h) {
        const { theme, className, isOnTop, groups } = this;
        if (!groups) {
            return null;
        }
        const groupElements = this.groups.map(this.renderGroup);
        const classNames = getClassNames(this.styles, { theme: theme, className, isOnTop, groups });
        return (<nav role="navigation" class={classNames.root}>
        {groupElements}
      </nav>);
    }
    renderGroup(group, groupIndex) {
        const { groups, theme } = this;
        const classNames = getClassNames(this.styles, {
            theme: theme,
            isGroup: true,
            isExpanded: this.isGroupExpanded(group),
            groups,
        });
        return (<div key={groupIndex}>
        <div class={classNames.groupContent}>{this.renderLinks(group.links, 0)}</div>
      </div>);
    }
    renderLinks(links, nestingLevel) {
        if (!links || !links.length)
            return null;
        const linkElements = links.map((link, linkIndex) => this.renderLink(link, linkIndex, nestingLevel));
        const { groups, theme } = this;
        const classNames = getClassNames(this.styles, { theme: theme, groups });
        return (<ul role="list" class={classNames.navItems}>
        {linkElements}
      </ul>);
    }
    renderLink(link, linkIndex, nestingLevel) {
        const { groups, theme } = this;
        const classNames = getClassNames(this.styles, { theme: theme, groups });
        return (<li key={linkIndex} role="listitem" class={classNames.navItem}>
        {this.renderCompositeLink(link, linkIndex, nestingLevel)}
        {link.isExpanded ? this.renderLinks(link.links, ++nestingLevel) : null}
      </li>);
    }
    renderCompositeLink(link, linkIndex, nestingLevel) {
        const { styles, groups, theme } = this;
        const classNames = getClassNames(this.styles, {
            theme: theme,
            isExpanded: !!link.isExpanded,
            isSelected: false,
            isLink: true,
            isDisabled: link.disabled,
            position: INDENTATION_SIZE * nestingLevel + 1,
            groups,
        });
        return (<div key={linkIndex} class={classNames.compositeLink}>
        {link.links && link.links.length > 0 ? (<button onClick={this.onLinkExpandClicked.bind(this, link)} class={classNames.chevronButton}>
            <Icon class-name={classNames.chevronIcon} 
        // @ts-ignore
        style={link.isExpanded && { transform: 'rotate(-180deg)' }} icon-name="ChevronDown"/>
          </button>) : null}
        {this.renderNavLink(link, linkIndex, nestingLevel)}
      </div>);
    }
    onLinkExpandClicked(link) {
        link.isExpanded = !link.isExpanded;
    }
    renderNavLink(link, linkIndex, nestingLevel) {
        const isSelected = link.key === this.internalSelectedKey;
        const isLinkWithIcon = link.icon || link.iconProps;
        const { groups, theme } = this;
        const classNames = getClassNames(this.styles, {
            theme: theme,
            isSelected: isSelected,
            isDisabled: link.disabled,
            isButtonEntry: link.onClick && !link.forceAnchor,
            leftPadding: INDENTATION_SIZE * nestingLevel + BASE_INDENT + (isLinkWithIcon ? 0 : 24),
            groups,
        });
        return (<ActionButton 
        // @ts-ignore
        href={link.href || (link.forceAnchor ? '#' : undefined)} title={link.title || link.name} target={link.target} disabled={link.disabled} nativeOnClick={this.onNavLinkClicked.bind(this, link, linkIndex)} class={[classNames.link, isSelected && classNames.selected]} style={{ paddingLeft: `${INDENTATION_SIZE * nestingLevel + BASE_INDENT + (isLinkWithIcon ? 0 : 24)}px` }}>
        {link.name}
      </ActionButton>);
    }
    onNavLinkClicked(link, linkIndex) {
        if (link.onClick)
            link.onClick(link);
        if (!link.url && link.links && link.links.length > 0) {
            link.isExpanded = !link.isExpanded;
        }
        this.internalSelectedKey = link.key;
    }
    preventBounce(link, ev) {
        if (!link.href && link.forceAnchor)
            ev.preventDefault();
    }
    isGroupExpanded(group) {
        if (group.name && this.isGroupCollapsed.hasOwnProperty(group.name)) {
            return !this.isGroupCollapsed[group.name];
        }
        if (group.collapseByDefault !== undefined) {
            return !group.collapseByDefault;
        }
        return true;
    }
    toggleCollapsed(group) {
        if (group.name) {
            const newGroupCollapsed = {
                ...this.isGroupCollapsed,
                [group.name]: this.isGroupExpanded(group),
            };
            this.$set(this, 'isGroupCollapsed', newGroupCollapsed);
        }
    }
};
__decorate([
    Prop({ type: Array, default: () => [] }),
    __metadata("design:type", Array)
], Nav.prototype, "groups", void 0);
__decorate([
    Prop({ type: String, default: '' }),
    __metadata("design:type", String)
], Nav.prototype, "selectedKey", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], Nav.prototype, "isOnTop", void 0);
Nav = __decorate([
    Component({
        components: { ActionButton, Icon },
    })
], Nav);
export default Nav;
//# sourceMappingURL=Nav.jsx.map