import { __decorate, __metadata } from "tslib";
import { Component, Prop, Watch } from 'vue-property-decorator';
import { PanelType } from './Panel.types';
import BaseComponent from '../BaseComponent';
import { Layer } from '../Layer';
import { Overlay } from '../Overlay';
import { IconButton } from '../Button';
import { classNamesFunction, getId, elementContains, getRTL } from '@uifabric-vue/utilities';
import { IconFontSizes } from '@uifabric/styling';
const getClassNames = classNamesFunction();
var PanelVisibilityState;
(function (PanelVisibilityState) {
    PanelVisibilityState[PanelVisibilityState["closed"] = 0] = "closed";
    PanelVisibilityState[PanelVisibilityState["animatingOpen"] = 1] = "animatingOpen";
    PanelVisibilityState[PanelVisibilityState["open"] = 2] = "open";
    PanelVisibilityState[PanelVisibilityState["animatingClosed"] = 3] = "animatingClosed";
})(PanelVisibilityState || (PanelVisibilityState = {}));
let PanelBase = class PanelBase extends BaseComponent {
    constructor() {
        super(...arguments);
        this.IconFontSizes = IconFontSizes;
        this.isFooterSticky = false;
        this.id = getId('Panel');
        this.visibility = PanelVisibilityState.closed;
        this.animationCallback = null;
    }
    created() {
        window.addEventListener('resize', this.updateFooterPosition);
        if (this.shouldListenForOuterClick) {
            window.addEventListener('mousedown', this.dismissOnOuterClick, true);
        }
    }
    onIsOpenUpdated(newVal, prevVal) {
        console.log('onIsOpenUpdated');
        if (newVal) {
            this.visibility = PanelVisibilityState.animatingOpen;
        }
    }
    onVisibilityUpdated(newVal, prevVal) {
        console.log(newVal, prevVal);
        if (newVal !== prevVal) {
            this.clearExistingAnimationTimer();
            if (newVal === PanelVisibilityState.animatingOpen) {
                this.animateTo(PanelVisibilityState.open);
            }
            else if (newVal === PanelVisibilityState.animatingClosed) {
                this.animateTo(PanelVisibilityState.closed);
            }
        }
    }
    get shouldListenForOuterClick() {
        return !!this.isBlocking && !!this.isOpen;
    }
    get classNames() {
        const { styles, theme, className = '', focusTrapZoneProps, hasCloseButton, headerClassName = '', isFooterSticky, isFooterAtBottom, isHiddenOnDismiss, type, } = this;
        const { visibility } = this;
        const isOpen = this.isActive;
        const isAnimating = visibility === PanelVisibilityState.animatingClosed || visibility === PanelVisibilityState.animatingOpen;
        const isLeft = !!(type === PanelType.smallFixedNear || type === PanelType.customNear);
        const isRTL = getRTL(theme);
        const isOnRightSide = isRTL ? isLeft : !isLeft;
        return getClassNames(styles, {
            theme: theme,
            className,
            focusTrapZoneClassName: focusTrapZoneProps ? focusTrapZoneProps.className : undefined,
            hasCloseButton,
            headerClassName,
            isAnimating,
            isFooterSticky,
            isFooterAtBottom,
            isOnRightSide,
            isOpen,
            isHiddenOnDismiss,
            type,
            hasCustomNavigation: false,
        });
    }
    get isAnimating() {
        const { visibility } = this;
        return visibility === PanelVisibilityState.animatingClosed || visibility === PanelVisibilityState.animatingOpen;
    }
    get isActive() {
        return this.visibility === PanelVisibilityState.open || this.visibility === PanelVisibilityState.animatingOpen;
    }
    close() {
        console.log('close');
        console.log(this.isOpen, this.isActive, this.visibility);
        if (this.isOpen !== undefined)
            return;
        if (!this.isActive)
            return;
        console.log(this.visibility);
        this.visibility = PanelVisibilityState.animatingClosed;
    }
    dismiss(ev) {
        console.log('dismiss');
        if (this.onDismiss) {
            this.onDismiss(ev);
        }
        if (!ev || (ev && !ev.defaultPrevented)) {
            this.close();
        }
    }
    dismissOnOuterClick(ev) {
        const panel = this.$refs.panel;
        if (this.isActive && panel && !ev.defaultPrevented()) {
            if (!elementContains(panel, ev.target)) {
                if (this.onOuterClick) {
                    this.onOuterClick();
                    ev.preventDefault();
                }
                else {
                    this.dismiss();
                }
            }
        }
    }
    updateFooterPosition() {
        const scrollableContent = this.$refs.scrollableContent;
        if (scrollableContent) {
            const height = scrollableContent.clientHeight;
            const innerHeight = scrollableContent.scrollHeight;
            this.isFooterSticky = height < innerHeight;
        }
    }
    animateTo(newVisibilityState) {
        console.log('animateTo', newVisibilityState);
        this.animationCallback = this._async.setTimeout(() => {
            this.visibility = newVisibilityState;
            this._onTransitionComplete();
        }, 200);
    }
    clearExistingAnimationTimer() {
        if (this.animationCallback !== null) {
            this._async.clearTimeout(this.animationCallback);
        }
    }
    _onTransitionComplete() {
        this.updateFooterPosition();
        if (this.visibility === PanelVisibilityState.open && this.onOpened) {
            this.onOpened();
        }
        if (this.visibility === PanelVisibilityState.closed && this.onDismissed) {
            this.onDismissed();
        }
    }
    render() {
        const { classNames, isOpen, isAnimating, isHiddenOnDismiss, isBlocking, theme, headerText } = this;
        if (!isOpen && !isAnimating && !isHiddenOnDismiss)
            return;
        return (<Layer>
        <div ref="panel" class={classNames.root}>
          {(isBlocking && isOpen) && (<Overlay class={classNames.overlay}/>)}
          <div class={classNames.main}>
            <div class={classNames.commands}>
              <div class={classNames.navigation}>
                <IconButton class={classNames.closeButton} styles={{
            root: {
                height: 'auto',
                width: '44px',
                color: theme.palette.neutralSecondary,
                fontSize: IconFontSizes.large,
            },
            rootHovered: {
                color: theme.palette.neutralPrimary,
            },
        }} icon-props={{ iconName: 'Cancel' }} nativeOnClick={this.dismiss}/>
              </div>
            </div>

            <div class={classNames.contentInner}>
              <div class={classNames.header}>
                <p class={classNames.headerText}>
                  {this.$slots.header || headerText}
                </p>
              </div>

              <div ref="scrollableContent" class={classNames.scrollableContent}>
                <div class={classNames.content}>
                  {this.$slots.default}
                </div>
              </div>
            </div>
          </div>
        </div>
      </Layer>);
    }
};
__decorate([
    Prop({ type: String, default: null }),
    __metadata("design:type", String)
], PanelBase.prototype, "headerText", void 0);
__decorate([
    Prop(),
    __metadata("design:type", Object)
], PanelBase.prototype, "focusTrapZoneProps", void 0);
__decorate([
    Prop(),
    __metadata("design:type", Object)
], PanelBase.prototype, "headerClassName", void 0);
__decorate([
    Prop({ type: Number, default: PanelType.smallFixedFar }),
    __metadata("design:type", Number)
], PanelBase.prototype, "type", void 0);
__decorate([
    Prop({ type: Function, default: null }),
    __metadata("design:type", Function)
], PanelBase.prototype, "onDismiss", void 0);
__decorate([
    Prop({ type: Function, default: null }),
    __metadata("design:type", Function)
], PanelBase.prototype, "onDismissed", void 0);
__decorate([
    Prop({ type: Function, default: null }),
    __metadata("design:type", Function)
], PanelBase.prototype, "onOpened", void 0);
__decorate([
    Prop({ type: Function, default: null }),
    __metadata("design:type", Function)
], PanelBase.prototype, "onOuterClick", void 0);
__decorate([
    Prop({ type: Boolean, default: true }),
    __metadata("design:type", Boolean)
], PanelBase.prototype, "hasCloseButton", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], PanelBase.prototype, "isFooterAtBottom", void 0);
__decorate([
    Prop({ type: Boolean, default: null }),
    __metadata("design:type", Boolean)
], PanelBase.prototype, "isOpen", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], PanelBase.prototype, "isHiddenOnDismiss", void 0);
__decorate([
    Prop({ type: Boolean, default: true }),
    __metadata("design:type", Boolean)
], PanelBase.prototype, "isBlocking", void 0);
__decorate([
    Watch('isOpen'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean, Boolean]),
    __metadata("design:returntype", void 0)
], PanelBase.prototype, "onIsOpenUpdated", null);
__decorate([
    Watch('visibility'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, Number]),
    __metadata("design:returntype", void 0)
], PanelBase.prototype, "onVisibilityUpdated", null);
PanelBase = __decorate([
    Component
], PanelBase);
export { PanelBase };
//# sourceMappingURL=Panel.base.jsx.map