import { __decorate, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import { classNamesFunction, getRTL } from '@uifabric-vue/utilities';
import { ScrollbarVisibility } from './ScrollablePane.types';
import BaseComponent from '../BaseComponent';
const getClassNames = classNamesFunction();
let ScrollablePaneBase = class ScrollablePaneBase extends BaseComponent {
    constructor() {
        super(...arguments);
        this.stickyTopHeight = 0;
        this.stickyBottomHeight = 0;
        this.scrollbarWidth = 0;
        this.scrollbarHeight = 0;
        this.subscribers = new Set();
        this.stickies = new Set();
        this.mutationObserver = null;
    }
    created() {
        this.notifyThrottled = this._async.throttle(this.notifySubscribers, 50);
    }
    mounted() {
        this.events.on(this.$refs.contentContainer, 'scroll', this._onScroll);
        this.events.on(window, 'resize', this._onWindowResize);
        const { initialScrollPosition } = this;
        if (this.$refs.contentContainer && initialScrollPosition) {
            this.$refs.contentContainer.scrollTop = initialScrollPosition;
        }
        // Set sticky distances from top property, then sort in correct order and notify subscribers
        this.setStickiesDistanceFromTop();
        this.stickies.forEach(sticky => {
            this.sortSticky(sticky);
        });
        this.notifySubscribers();
        if ('MutationObserver' in window) {
            this.mutationObserver = new MutationObserver(mutation => {
                // Function to check if mutation is occuring in stickyAbove or stickyBelow
                function checkIfMutationIsSticky(mutationRecord) {
                    if (this.$refs.stickyAbove !== null && this.$refs.stickyBelow !== null) {
                        return this.$refs.stickyAbove.contains(mutationRecord.target) || this.$refs.stickyBelow.contains(mutationRecord.target);
                    }
                    return false;
                }
                // Compute the scrollbar height, which might have changed if the content's width changed and caused overflow
                const scrollbarHeight = this._getScrollbarHeight();
                // If the scrollbar height changed, update state so it's postioned correctly below sticky footer
                if (scrollbarHeight !== this.scrollbarHeight) {
                    this.scrollbarHeight = scrollbarHeight;
                }
                // Notify subscribers again to re-check whether Sticky should be Sticky'd or not
                this.notifySubscribers();
                // If mutation occurs in sticky header or footer, then update sticky top/bottom heights
                if (mutation.some(checkIfMutationIsSticky.bind(this))) {
                    this.updateStickyRefHeights();
                }
                else {
                    // If mutation occurs in scrollable region, then find Sticky it belongs to and force update
                    const stickyList = [];
                    this.stickies.forEach(sticky => {
                        if (sticky.$refs.root && sticky.$refs.root.contains(mutation[0].target)) {
                            stickyList.push(sticky);
                        }
                    });
                    if (stickyList.length) {
                        stickyList.forEach(sticky => {
                            sticky.$forceUpdate();
                        });
                    }
                }
            });
            if (this.$refs.root) {
                this.mutationObserver.observe(this.$refs.root, {
                    childList: true,
                    attributes: true,
                    subtree: true,
                    characterData: true,
                });
            }
        }
    }
    get classNames() {
        const { className, theme, styles } = this;
        return getClassNames(styles, {
            theme: theme,
            className,
            scrollbarVisibility: this.scrollbarVisibility,
        });
    }
    setStickiesDistanceFromTop() {
        if (this.$refs.contentContainer) {
            this.stickies.forEach((sticky) => {
                sticky.setDistanceFromTop(this.$refs.contentContainer);
            });
        }
    }
    forceLayoutUpdate() {
        this._onWindowResize();
    }
    subscribe(handler) {
        this.subscribers.add(handler);
    }
    ;
    unsubscribe(handler) {
        this.subscribers.delete(handler);
    }
    ;
    addSticky(sticky) {
        this.stickies.add(sticky);
        // If ScrollablePane is mounted, then sort sticky in correct place
        if (this.$refs.contentContainer) {
            sticky.setDistanceFromTop(this.$refs.contentContainer);
            this.sortSticky(sticky);
        }
    }
    ;
    removeSticky(sticky) {
        this.stickies.delete(sticky);
        this._removeStickyFromContainers(sticky);
        this.notifySubscribers();
    }
    ;
    sortSticky(sticky, sortAgain) {
        if (this.$refs.stickyAbove && this.$refs.stickyBelow) {
            if (sortAgain) {
                this._removeStickyFromContainers(sticky);
            }
            if (sticky.canStickyTop && sticky.$refs.stickyContentTop) {
                this._addToStickyContainer(sticky, this.$refs.stickyAbove, sticky.$refs.stickyContentTop);
            }
            if (sticky.canStickyBottom && sticky.$refs.stickyContentBottom) {
                this._addToStickyContainer(sticky, this.$refs.stickyBelow, sticky.$refs.stickyContentBottom);
            }
        }
    }
    ;
    updateStickyRefHeights() {
        const stickyItems = this.stickies;
        let stickyTopHeight = 0;
        let stickyBottomHeight = 0;
        stickyItems.forEach((sticky) => {
            const { isStickyTop, isStickyBottom } = sticky;
            if (sticky.$refs.nonStickyContent) {
                if (isStickyTop) {
                    stickyTopHeight += sticky.$refs.nonStickyContent.offsetHeight;
                }
                if (isStickyBottom) {
                    stickyBottomHeight += sticky.$refs.nonStickyContent.offsetHeight;
                }
                this._checkStickyStatus(sticky);
            }
        });
        this.stickyTopHeight = stickyTopHeight;
        this.stickyBottomHeight = stickyBottomHeight;
    }
    ;
    notifySubscribers() {
        if (this.$refs.contentContainer) {
            this.subscribers.forEach(handle => {
                // this.$refs.stickyBelow is passed in for calculating distance to determine Sticky status
                handle(this.$refs.contentContainer, this.$refs.stickyBelow);
            });
        }
    }
    ;
    getScrollPosition() {
        if (this.$refs.contentContainer) {
            return this.$refs.contentContainer.scrollTop;
        }
        return 0;
    }
    ;
    syncScrollSticky(sticky) {
        if (sticky && this.$refs.contentContainer) {
            sticky.syncScroll(this.$refs.contentContainer);
        }
    }
    ;
    _checkStickyStatus(sticky) {
        if (this.$refs.stickyAbove && this.$refs.stickyBelow && this.$refs.contentContainer && sticky.$refs.nonStickyContent) {
            // If sticky is sticky, then append content to appropriate container
            if (sticky.isStickyTop || sticky.isStickyBottom) {
                if (sticky.isStickyTop && !this.$refs.stickyAbove.contains(sticky.$refs.nonStickyContent) && sticky.$refs.stickyContentTop) {
                    sticky.addSticky(sticky.$refs.stickyContentTop);
                }
                if (sticky.isStickyBottom && !this.$refs.stickyBelow.contains(sticky.$refs.nonStickyContent) && sticky.$refs.stickyContentBottom) {
                    sticky.addSticky(sticky.$refs.stickyContentBottom);
                }
            }
            else if (!this.$refs.contentContainer.contains(sticky.$refs.nonStickyContent)) {
                // Reset sticky if it's not sticky and not in the contentContainer element
                sticky.resetSticky();
            }
        }
    }
    _addToStickyContainer(sticky, stickyContainer, stickyContentToAdd) {
        // If there's no children, append child to list, otherwise, sort though array and append at correct position
        if (!stickyContainer.children.length) {
            stickyContainer.appendChild(stickyContentToAdd);
        }
        else {
            // If stickyContentToAdd isn't a child element of target container, then append
            if (!stickyContainer.contains(stickyContentToAdd)) {
                const stickyChildrenElements = [].slice.call(stickyContainer.children);
                const stickyList = [];
                // Get stickies.  Filter by canStickyTop/Bottom, then sort by distance from top, and then
                // filter by elements that are in the stickyContainer already.
                this.stickies.forEach((stickyItem) => {
                    if (stickyContainer === this.$refs.stickyAbove && sticky.canStickyTop) {
                        stickyList.push(stickyItem);
                    }
                    else if (sticky.canStickyBottom) {
                        stickyList.push(stickyItem);
                    }
                });
                const stickyListSorted = stickyList
                    .sort((a, b) => {
                    return (a.distanceFromTop || 0) - (b.distanceFromTop || 0);
                })
                    .filter(item => {
                    const stickyContent = stickyContainer === this.$refs.stickyAbove ? item.stickyContentTop : item.stickyContentBottom;
                    if (stickyContent) {
                        return stickyChildrenElements.indexOf(stickyContent) > -1;
                    }
                });
                // Get first element that has a distance from top that is further than our sticky that is being added
                let targetStickyToAppendBefore;
                for (const stickyListItem of stickyListSorted) {
                    if ((stickyListItem.distanceFromTop || 0) >= (sticky.distanceFromTop || 0)) {
                        targetStickyToAppendBefore = stickyListItem;
                        break;
                    }
                }
                // If target element to append before is known, then grab respective stickyContentTop/Bottom element and insert before
                let targetContainer = null;
                if (targetStickyToAppendBefore) {
                    targetContainer =
                        stickyContainer === this.$refs.stickyAbove
                            ? targetStickyToAppendBefore.stickyContentTop
                            : targetStickyToAppendBefore.stickyContentBottom;
                }
                stickyContainer.insertBefore(stickyContentToAdd, targetContainer);
            }
        }
    }
    ;
    _removeStickyFromContainers(sticky) {
        if (this.$refs.stickyAbove && sticky.stickyContentTop && this.$refs.stickyAbove.contains(sticky.stickyContentTop)) {
            this.$refs.stickyAbove.removeChild(sticky.stickyContentTop);
        }
        if (this.$refs.stickyBelow && sticky.stickyContentBottom && this.$refs.stickyBelow.contains(sticky.stickyContentBottom)) {
            this.$refs.stickyBelow.removeChild(sticky.stickyContentBottom);
        }
    }
    ;
    _onWindowResize() {
        const scrollbarWidth = this._getScrollbarWidth();
        const scrollbarHeight = this._getScrollbarHeight();
        this.scrollbarWidth = scrollbarWidth;
        this.scrollbarHeight = scrollbarHeight;
        this.notifySubscribers();
    }
    ;
    getStickyContainerStyle(height, isTop) {
        return {
            height: height,
            ...(getRTL(this.theme)
                ? {
                    right: '0',
                    left: `${this.scrollbarWidth || this._getScrollbarWidth() || 0}px`,
                }
                : {
                    left: '0',
                    right: `${this.scrollbarWidth || this._getScrollbarWidth() || 0}px`,
                }),
            ...(isTop
                ? {
                    top: '0',
                }
                : {
                    bottom: `${this.scrollbarHeight || this._getScrollbarHeight() || 0}px`,
                }),
        };
    }
    ;
    _getScrollbarWidth() {
        const { contentContainer } = this.$refs;
        return contentContainer ? contentContainer.offsetWidth - contentContainer.clientWidth : 0;
    }
    _getScrollbarHeight() {
        const { contentContainer } = this.$refs;
        return contentContainer ? contentContainer.offsetHeight - contentContainer.clientHeight : 0;
    }
    _onScroll() {
        const { contentContainer } = this.$refs;
        if (contentContainer) {
            this.stickies.forEach((sticky) => {
                // @ts-ignore
                sticky.syncScroll(contentContainer);
            });
        }
        this.notifyThrottled();
    }
    ;
    render() {
        const { classNames, stickyBottomHeight, stickyTopHeight } = this;
        return (<div ref="root" class={classNames.root}>
        <div ref="stickyAbove" aria-hidden="true" class={classNames.stickyAbove} style={this.getStickyContainerStyle(stickyTopHeight, true)}/>

        <div ref="contentContainer" class={classNames.contentContainer} data-is-scrollable="true">
          {this.$slots.default}
        </div>

        <div aria-hidden="true" class={classNames.stickyBelow} style={this.getStickyContainerStyle(stickyBottomHeight, false)}>
          <div ref="stickyBelow" class={classNames.stickyBelowItems}/>
        </div>
      </div>);
    }
};
__decorate([
    Prop({ type: Number, default: 0 }),
    __metadata("design:type", Number)
], ScrollablePaneBase.prototype, "initialScrollPosition", void 0);
__decorate([
    Prop({ type: String, default: ScrollbarVisibility.auto }),
    __metadata("design:type", String)
], ScrollablePaneBase.prototype, "scrollbarVisibility", void 0);
ScrollablePaneBase = __decorate([
    Component({
        components: {},
        provide() {
            return {
                context: {
                    scrollablePane: {
                        subscribe: this.subscribe,
                        unsubscribe: this.unsubscribe,
                        addSticky: this.addSticky,
                        removeSticky: this.removeSticky,
                        updateStickyRefHeights: this.updateStickyRefHeights,
                        sortSticky: this.sortSticky,
                        notifySubscribers: this.notifySubscribers,
                        syncScrollSticky: this.syncScrollSticky,
                    },
                },
            };
        },
    })
], ScrollablePaneBase);
export { ScrollablePaneBase };
//# sourceMappingURL=ScrollablePane.base.jsx.map