import { __decorate, __metadata } from "tslib";
import { Component, Prop } from 'vue-property-decorator';
import { Label } from '../Label/';
import BaseComponent from '../BaseComponent';
import { KeyCodes, classNamesFunction } from '@uifabric-vue/utilities';
const getClassNames = classNamesFunction();
export const ONKEYDOWN_TIMEOUT_DURATION = 1000;
let SliderBase = class SliderBase extends BaseComponent {
    constructor() {
        super(...arguments);
        this.internalValue = this.value || this.defaultValue || this.min;
        this.renderedValue = this.value || this.defaultValue || this.min;
        this.onKeyDownTimer = -1;
    }
    get classNames() {
        const { className, disabled, vertical, renderedValue, internalValue, showValue, theme } = this;
        return getClassNames(this.styles, {
            className,
            disabled,
            vertical,
            showTransitions: renderedValue === internalValue,
            showValue,
            theme: theme,
        });
    }
    get thumbOffsetPercent() {
        const { min, max, renderedValue } = this;
        return min === max ? 0 : ((renderedValue - min) / (max - min)) * 100;
    }
    get zeroOffsetPercent() {
        const { min, max } = this;
        return min >= 0 ? 0 : (-min / (max - min)) * 100;
    }
    get lengthString() {
        return this.vertical ? 'height' : 'width';
    }
    onMouseDown(event) {
        if (this.disabled)
            return;
        if (event.type === 'mousedown') {
            window.addEventListener('mousemove', this.onMove, true);
            window.addEventListener('mouseup', this.onMouseUp, true);
        }
        else if (event.type === 'touchstart') {
            window.addEventListener('touchmove', this.onMove, true);
            window.addEventListener('touchend', this.onMouseUp, true);
        }
        this.onMove(event);
    }
    onMouseUp(event) {
        if (this.disabled)
            return;
        this.renderedValue = this.internalValue;
        window.removeEventListener('mousemove', this.onMove, true);
        window.removeEventListener('mouseup', this.onMouseUp, true);
        window.removeEventListener('touchmove', this.onMove, true);
        window.removeEventListener('touchend', this.onMouseUp, true);
    }
    onMove(event) {
        window.requestAnimationFrame(() => {
            const { max, min, step, vertical } = this;
            const steps = (max - min) / step;
            const sliderPositionRect = this.$refs.sliderLine.getBoundingClientRect();
            const sliderLength = !vertical
                ? sliderPositionRect.width
                : sliderPositionRect.height;
            const stepLength = sliderLength / steps;
            let currentSteps;
            let distance;
            if (!vertical) {
                const left = this.getPosition(event, vertical);
                distance = left - sliderPositionRect.left;
                currentSteps = distance / stepLength;
            }
            else {
                const bottom = this.getPosition(event, vertical);
                distance = sliderPositionRect.bottom - bottom;
                currentSteps = distance / stepLength;
            }
            let internalValue;
            let renderedValue;
            // The value shouldn't be bigger than max or be smaller than min.
            if (currentSteps > Math.floor(steps)) {
                renderedValue = internalValue = max;
            }
            else if (currentSteps < 0) {
                renderedValue = internalValue = min;
            }
            else {
                renderedValue = min + step * currentSteps;
                internalValue = min + step * Math.round(currentSteps);
            }
            this.updateValue(internalValue, renderedValue);
        });
    }
    getPosition(event, vertical) {
        let currentPosition;
        switch (event.type) {
            case 'mousedown':
            case 'mousemove':
                currentPosition = !vertical
                    ? event.clientX
                    : event.clientY;
                break;
            case 'touchstart':
            case 'touchmove':
                currentPosition = !vertical
                    ? event.touches[0].clientX
                    : event.touches[0].clientY;
                break;
        }
        return currentPosition;
    }
    updateValue(value, renderedValue) {
        const { step, snapToStep } = this;
        let numDec = 0;
        if (isFinite(step)) {
            while (Math.round(step * Math.pow(10, numDec)) / Math.pow(10, numDec) !== step) {
                numDec++;
            }
        }
        // Make sure value has correct number of decimal places based on number of decimals in step
        const roundedValue = parseFloat(value.toFixed(numDec));
        const valueChanged = roundedValue !== this.internalValue;
        if (snapToStep) {
            renderedValue = roundedValue;
        }
        this.internalValue = roundedValue;
        this.renderedValue = renderedValue;
        this.$emit('input', roundedValue);
    }
    onKeyDown(event) {
        let value = this.internalValue;
        const { max, min, step } = this;
        let diff = 0;
        switch (event.which) {
            case KeyCodes.left:
            case KeyCodes.down:
                diff = -step;
                this.clearOnKeyDownTimer();
                this.setOnKeyDownTimer(event);
                break;
            case KeyCodes.right:
            case KeyCodes.up:
                diff = step;
                this.clearOnKeyDownTimer();
                this.setOnKeyDownTimer(event);
                break;
            case KeyCodes.home:
                value = min;
                break;
            case KeyCodes.end:
                value = max;
                break;
            default:
                return;
        }
        const newValue = Math.min(max, Math.max(min, value + diff));
        this.updateValue(newValue, newValue);
        event.preventDefault();
        event.stopPropagation();
    }
    clearOnKeyDownTimer() {
        clearTimeout(this.onKeyDownTimer);
    }
    ;
    setOnKeyDownTimer(event) {
        this.onKeyDownTimer = setTimeout(() => {
        }, ONKEYDOWN_TIMEOUT_DURATION);
    }
    ;
    render() {
        const { classNames, disabled, originFromZero, vertical, zeroOffsetPercent, thumbOffsetPercent, lengthString, internalValue, css, label } = this;
        // TODO(code) `originFromZero` fragments/spans look ugly; maybe could be improved
        return (<div class={classNames.root}>
        <Label class={classNames.titleLabel}>{label}</Label>

        <div class={classNames.container}>
          <div class={classNames.slideBox} tabindex={disabled ? void 0 : 0} onMousedown={this.onMouseDown} onKeydown={this.onKeyDown}>
            <div ref="sliderLine" class={classNames.line}>
              {originFromZero && (<span class={classNames.zeroTick} style={{ [vertical ? 'bottom' : 'left']: `${zeroOffsetPercent}%` }}/>)}
              <span class={classNames.thumb} style={{ [vertical ? 'bottom' : 'left']: `${thumbOffsetPercent}%` }}/>

              {originFromZero && (<span class={css(classNames.lineContainer, classNames.inactiveSection)} style={{ [lengthString]: `${Math.min(thumbOffsetPercent, zeroOffsetPercent)}%` }}/>)}
              {originFromZero && (<span class={css(classNames.lineContainer, classNames.activeSection)} style={{ [lengthString]: `${Math.abs(zeroOffsetPercent - thumbOffsetPercent)}%` }}/>)}
              {originFromZero && (<span class={css(classNames.lineContainer, classNames.inactiveSection)} style={{ [lengthString]: `${Math.min(100 - thumbOffsetPercent, 100 - zeroOffsetPercent)}%` }}/>)}

              {!originFromZero && (<span class={css(classNames.lineContainer, classNames.activeSection)} style={{ [lengthString]: `${thumbOffsetPercent}%` }}/>)}
              {!originFromZero && (<span class={css(classNames.lineContainer, classNames.inactiveSection)} style={{ [lengthString]: `${100 - thumbOffsetPercent}%` }}/>)}
            </div>
          </div>

          <Label class={classNames.valueLabel}>
            {this.$scopedSlots.value
            ? this.$scopedSlots.value({ value: internalValue })
            : internalValue}
          </Label>
        </div>
      </div>);
    }
};
__decorate([
    Prop({ type: String, default: null }),
    __metadata("design:type", String)
], SliderBase.prototype, "label", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], SliderBase.prototype, "disabled", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], SliderBase.prototype, "vertical", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], SliderBase.prototype, "snapToStep", void 0);
__decorate([
    Prop({ type: Number, default: 0 }),
    __metadata("design:type", Number)
], SliderBase.prototype, "min", void 0);
__decorate([
    Prop({ type: Number, default: 10 }),
    __metadata("design:type", Number)
], SliderBase.prototype, "max", void 0);
__decorate([
    Prop({ type: Number, default: 1 }),
    __metadata("design:type", Number)
], SliderBase.prototype, "step", void 0);
__decorate([
    Prop({ type: Number, default: null }),
    __metadata("design:type", Number)
], SliderBase.prototype, "value", void 0);
__decorate([
    Prop({ type: Number, default: null }),
    __metadata("design:type", Number)
], SliderBase.prototype, "defaultValue", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], SliderBase.prototype, "showValue", void 0);
__decorate([
    Prop({ type: Boolean, default: false }),
    __metadata("design:type", Boolean)
], SliderBase.prototype, "originFromZero", void 0);
SliderBase = __decorate([
    Component({
        components: { Label },
    })
], SliderBase);
export { SliderBase };
//# sourceMappingURL=Slider.base.jsx.map