// import { any } from '../../Keytip'
import { EventGroup, getId } from '@uifabric-vue/utilities';
import { KeytipEvents } from './KeytipConstants';
/**
 * This class is responsible for handling registering, updating, and unregistering of keytips
 */
export class KeytipManager {
    constructor() {
        this.keytips = {};
        this.persistedKeytips = {};
        this.sequenceMapping = {};
        // This is (and should be) updated and kept in sync
        // with the inKeytipMode in KeytipLayer.
        this.inKeytipMode = false;
        // Boolean that gets checked before entering keytip mode by the KeytipLayer
        // Used for an override in special cases (e.g. Disable entering keytip mode when a modal is shown)
        this.shouldEnterKeytipMode = true;
        // Boolean to indicate whether to delay firing an event to update subscribers of
        // keytip data changed.
        this.delayUpdatingKeytipChange = false;
    }
    /**
     * Static function to get singleton KeytipManager instance
     *
     * @returns {KeytipManager} Singleton KeytipManager instance
     */
    static getInstance() {
        return this._instance;
    }
    /**
     * Initialization code to set set parameters to define
     * how the KeytipManager handles keytip data.
     *
     * @param delayUpdatingKeytipChange - T/F if we should delay notifiying keytip subscribers
     * of keytip changes
     */
    init(delayUpdatingKeytipChange) {
        this.delayUpdatingKeytipChange = delayUpdatingKeytipChange;
    }
    /**
     * Registers a keytip
     *
     * @param keytipProps - Keytip to register
     * @param persisted - T/F if this keytip should be persisted, default is false
     * @returns {string} Unique ID for this keytip
     */
    register(keytipProps, persisted = false) {
        let props = keytipProps;
        if (!persisted) {
            // Add the overflowSetSequence if necessary
            props = this.addParentOverflow(keytipProps);
            this.sequenceMapping[props.keySequences.toString()] = props;
        }
        // Create a unique keytip
        const uniqueKeytip = this._getUniqueKtp(props);
        // Add to dictionary
        persisted ? (this.persistedKeytips[uniqueKeytip.uniqueID] = uniqueKeytip) : (this.keytips[uniqueKeytip.uniqueID] = uniqueKeytip);
        // We only want to add something new if we are currently showing keytip mode
        if (this.inKeytipMode || !this.delayUpdatingKeytipChange) {
            const event = persisted ? KeytipEvents.PERSISTED_KEYTIP_ADDED : KeytipEvents.KEYTIP_ADDED;
            EventGroup.raise(this, event, {
                keytip: props,
                uniqueID: uniqueKeytip.uniqueID,
            });
        }
        return uniqueKeytip.uniqueID;
    }
    /**
     * Update a keytip
     *
     * @param keytipProps - Keytip to update
     * @param uniqueID - Unique ID of this keytip
     */
    update(keytipProps, uniqueID) {
        const newKeytipProps = this.addParentOverflow(keytipProps);
        const uniqueKeytip = this._getUniqueKtp(newKeytipProps, uniqueID);
        const oldKeyTip = this.keytips[uniqueID];
        if (oldKeyTip) {
            // Update everything except 'visible'
            uniqueKeytip.keytip.visible = oldKeyTip.keytip.visible;
            // Update keytip in this.keytips
            this.keytips[uniqueID] = uniqueKeytip;
            // Update the sequence to be up to date
            delete this.sequenceMapping[oldKeyTip.keytip.keySequences.toString()];
            this.sequenceMapping[uniqueKeytip.keytip.keySequences.toString()] = uniqueKeytip.keytip;
            // Raise event only if we are currently in keytip mode
            if (this.inKeytipMode || !this.delayUpdatingKeytipChange) {
                EventGroup.raise(this, KeytipEvents.KEYTIP_UPDATED, {
                    keytip: uniqueKeytip.keytip,
                    uniqueID: uniqueKeytip.uniqueID,
                });
            }
        }
    }
    /**
     * Unregisters a keytip
     *
     * @param keytipToRemove - any of the keytip to remove
     * @param uniqueID - Unique ID of this keytip
     * @param persisted - T/F if this keytip should be persisted, default is false
     */
    unregister(keytipToRemove, uniqueID, persisted = false) {
        persisted ? delete this.persistedKeytips[uniqueID] : delete this.keytips[uniqueID];
        !persisted && delete this.sequenceMapping[keytipToRemove.keySequences.toString()];
        const event = persisted ? KeytipEvents.PERSISTED_KEYTIP_REMOVED : KeytipEvents.KEYTIP_REMOVED;
        // Update keytips only if we're in keytip mode
        if (this.inKeytipMode || !this.delayUpdatingKeytipChange) {
            EventGroup.raise(this, event, {
                keytip: keytipToRemove,
                uniqueID: uniqueID,
            });
        }
    }
    /**
     * Manual call to enter keytip mode
     */
    enterKeytipMode() {
        EventGroup.raise(this, KeytipEvents.ENTER_KEYTIP_MODE);
    }
    /**
     * Manual call to exit keytip mode
     */
    exitKeytipMode() {
        EventGroup.raise(this, KeytipEvents.EXIT_KEYTIP_MODE);
    }
    /**
     * Gets all any from this.keytips
     *
     * @returns {any[]} All keytips stored in the manager
     */
    getKeytips() {
        return Object.keys(this.keytips).map(key => this.keytips[key].keytip);
    }
    /**
     * Adds the overflowSetSequence to the keytipProps if its parent keytip also has it
     *
     * @param keytipProps - Keytip props to add overflowSetSequence to if necessary
     * @returns {any} - Modified keytip props, if needed to be modified
     */
    addParentOverflow(keytipProps) {
        const fullSequence = [...keytipProps.keySequences];
        fullSequence.pop();
        if (fullSequence.length !== 0) {
            const parentKeytip = this.sequenceMapping[fullSequence.toString()];
            if (parentKeytip && parentKeytip.overflowSetSequence) {
                return {
                    ...keytipProps,
                    overflowSetSequence: parentKeytip.overflowSetSequence,
                };
            }
        }
        return keytipProps;
    }
    /**
     * Public function to bind for overflow items that have a submenu
     *
     * @param overflowButtonSequences
     * @param keytipSequences
     */
    menuExecute(overflowButtonSequences, keytipSequences) {
        EventGroup.raise(this, KeytipEvents.PERSISTED_KEYTIP_EXECUTE, {
            overflowButtonSequences,
            keytipSequences,
        });
    }
    /**
     * Creates an IUniqueKeytip object
     *
     * @param keytipProps - any
     * @param uniqueID - Unique ID, will default to the next unique ID if not passed
     * @returns {IUniqueKeytip} IUniqueKeytip object
     */
    _getUniqueKtp(keytipProps, uniqueID = getId()) {
        return { keytip: { ...keytipProps }, uniqueID };
    }
}
KeytipManager._instance = new KeytipManager();
//# sourceMappingURL=KeytipManager.js.map